import streamlit as st
import pandas as pd
import numpy as np
from energy_tracker_core.Visulization.workspace_task import WorkSpace, Task
from scipy import interpolate

# 页面配置
st.set_page_config(
    page_title="结果对比",
    page_icon="📊",
    layout="wide"
)

st.title("📊 任务结果对比")
ws: WorkSpace = None
ws = st.session_state.get("workspace", None)

# 辅助函数：对数据进行插值处理
def interpolate_data(data_series, target_length):
    """
    对数据序列进行插值，使其长度达到目标长度
    """
    # 原始数据的索引
    orig_indices = np.arange(len(data_series))
    # 目标索引
    target_indices = np.linspace(0, len(data_series) - 1, target_length)
    # 创建插值函数
    if len(data_series) > 1:
        f = interpolate.interp1d(orig_indices, data_series, kind='linear')
        # 执行插值
        interpolated_data = f(target_indices)
        return interpolated_data
    else:
        # 如果只有一个数据点，无法进行插值，则复制该值
        return np.full(target_length, data_series.iloc[0])

# 检查前序工作
if 'workspace' not in st.session_state:
    st.warning("请先在 CSV 管理页面初始化工作目录。")
else:
    if 'edited_task_showframe' not in st.session_state:
        st.warning("请先在 CSV 管理页面至少选择两个要对比的csv文件。")
    else:
        # 获取所有选中的任务名称
        selected_mask = st.session_state.edited_task_showframe['is_selected'] == True
        selected_names = st.session_state.edited_task_showframe[selected_mask]['name'].tolist()
        
        if len(selected_names) < 2:
            st.warning("请至少选择两个任务进行对比。")
        else:
            # 获取对应的任务对象
            selected_tasks = [ws.get_task_by_name(name) for name in selected_names]
            
            # 确保所有任务都分析过
            for task in selected_tasks:
                if not hasattr(task, 'statistics') or not task.statistics:
                    task.analyse()
            
            # 创建对比数据
            comparison_data = {}
            for task in selected_tasks:
                task_name = task.basic_info['name']
                
                # 收集基本统计数据用于对比
                comparison_data[task_name] = {
                    '问答总数': task.statistics['问答总数'],
                    '总持续时间(秒)': task.statistics['总持续时间(秒)'],
                    '正确率': task.statistics['正确率'],
                    '平均每次问答时间(秒)': task.statistics['平均每次问答时间(秒)'],
                    'CPU总能耗(Wh)': task.statistics['CPU总能耗(Wh)'],
                    'GPU总能耗(Wh)': task.statistics['GPU总能耗(Wh)'],
                    '总能耗(Wh)': task.statistics['总能耗(Wh)'],
                    'CPU平均每次问答能耗(Wh)': task.statistics['CPU平均每次问答能耗(Wh)'],
                    'GPU平均每次问答能耗(Wh)': task.statistics['GPU平均每次问答能耗(Wh)'],
                    '平均每次问答总能耗(Wh)': task.statistics['平均每次问答总能耗(Wh)'],
                    'CPU平均功率(W)': task.statistics['CPU平均功率(W)'],
                    'GPU平均功率(W)': task.statistics['GPU平均功率(W)'],
                    '总平均功率(W)': task.statistics['总平均功率(W)'],
                }

            # 转换为DataFrame便于绘图
            comparison_df = pd.DataFrame(comparison_data).T
            
            # 提供不同对比维度的选择
            st.markdown("### 选择对比维度")
            comparison_tabs = st.tabs(["基本指标", "能耗对比", "功率对比", "组合对比"])
            
            with comparison_tabs[0]:
                st.markdown("#### 基本指标对比")
                basic_metrics = ['问答总数', '总持续时间(秒)', '正确率', '平均每次问答时间(秒)']
                selected_basic_metrics = st.multiselect(
                    "选择要对比的基本指标", 
                    basic_metrics,
                    default=['问答总数', '正确率']
                )
                
                if selected_basic_metrics:
                    st.bar_chart(comparison_df[selected_basic_metrics])
            
            with comparison_tabs[1]:
                st.markdown("#### 能耗对比")
                energy_metrics = ['CPU总能耗(Wh)', 'GPU总能耗(Wh)', '总能耗(Wh)', 
                               'CPU平均每次问答能耗(Wh)', 'GPU平均每次问答能耗(Wh)', '平均每次问答总能耗(Wh)']
                selected_energy_metrics = st.multiselect(
                    "选择要对比的能耗指标", 
                    energy_metrics,
                    default=['总能耗(Wh)', 'CPU总能耗(Wh)', 'GPU总能耗(Wh)']
                )
                
                if selected_energy_metrics:
                    # 绘制能耗对比图
                    st.bar_chart(comparison_df[selected_energy_metrics])
                    
                    # 提供数据表格查看
                    with st.expander("查看详细数据"):
                        st.dataframe(comparison_df[selected_energy_metrics])
            
            with comparison_tabs[2]:
                st.markdown("#### 功率对比")
                power_metrics = ['CPU平均功率(W)', 'GPU平均功率(W)', '总平均功率(W)']
                selected_power_metrics = st.multiselect(
                    "选择要对比的功率指标", 
                    power_metrics,
                    default=power_metrics
                )
                
                if selected_power_metrics:
                    # 绘制功率对比图
                    st.bar_chart(comparison_df[selected_power_metrics])
                    
                    # 提供数据表格查看
                    with st.expander("查看详细数据"):
                        st.dataframe(comparison_df[selected_power_metrics])
            
            with comparison_tabs[3]:
                st.markdown("#### 自定义组合对比")
                all_metrics = basic_metrics + energy_metrics + power_metrics
                custom_metrics = st.multiselect(
                    "选择要对比的指标", 
                    all_metrics,
                    default=['总能耗(Wh)', '总平均功率(W)', '正确率']
                )
                
                chart_type = st.radio("选择图表类型", ["柱状图", "折线图"], horizontal=True)
                
                if custom_metrics:
                    if chart_type == "柱状图":
                        st.bar_chart(comparison_df[custom_metrics])
                    else:
                        st.line_chart(comparison_df[custom_metrics])
                    
                    # 提供数据表格查看
                    with st.expander("查看详细数据"):
                        st.dataframe(comparison_df[custom_metrics])
            
            # 高级对比：时间序列叠加对比
            st.markdown("### 能耗趋势对比")
            trend_tabs = st.tabs(["累计能耗趋势", "单次能耗分布", "实时功率变化"])
            
            with trend_tabs[0]:
                st.markdown("#### 累计能耗趋势对比")
                
                # 设置所有任务统一的标准化点数（100点足够展示趋势）
                standard_points = 100
                normalize_method = st.radio(
                    "标准化方法",
                    ["百分比进度", "插值到相同点数"],
                    horizontal=True,
                    help="百分比进度：按任务进度百分比对齐；插值到相同点数：将所有任务重采样到相同点数"
                )
                
                # 创建一个DataFrame存储所有任务的累计能耗
                energy_trend_data = pd.DataFrame()
                
                for task in selected_tasks:
                    task_name = task.basic_info['name']
                    # 计算累计能耗
                    total_cumsum = task.data['total_incremental_energy'].cumsum()
                    
                    if normalize_method == "百分比进度":
                        # 创建百分比进度索引
                        progress_pct = np.linspace(0, 100, len(total_cumsum))
                        # 添加到DataFrame
                        task_df = pd.DataFrame({task_name: total_cumsum.values}, index=progress_pct)
                        energy_trend_data = pd.concat([energy_trend_data, task_df], axis=1)
                    else:
                        # 对累计能耗数据进行插值，使所有任务具有相同点数
                        interpolated_cumsum = interpolate_data(total_cumsum, standard_points)
                        # 使用统一的索引添加到DataFrame
                        if energy_trend_data.empty:
                            energy_trend_data = pd.DataFrame(index=range(standard_points))
                        energy_trend_data[task_name] = interpolated_cumsum
                
                # 绘制累计能耗对比图
                st.line_chart(energy_trend_data)
                
                with st.expander("查看图表说明"):
                    if normalize_method == "百分比进度":
                        st.markdown("""
                        **累计能耗趋势对比说明：**
                        - X轴代表任务进度百分比，从0%到100%
                        - Y轴代表累计能耗，单位为Wh
                        - 每条线代表一个选中的任务
                        - 斜率越大的部分，表示该阶段能耗增长越快
                        """)
                    else:
                        st.markdown("""
                        **累计能耗趋势对比说明：**
                        - X轴代表标准化的数据点序号（通过插值将不同长度的任务调整为相同点数）
                        - Y轴代表累计能耗，单位为Wh
                        - 每条线代表一个选中的任务
                        - 斜率越大的部分，表示该阶段能耗增长越快
                        - 通过插值处理，可以直接比较不同长度任务的趋势
                        """)
            
            with trend_tabs[1]:
                st.markdown("#### 单次能耗分布对比")
                
                # 单次能耗分布的标准化设置
                st.write("##### 数据标准化设置")
                energy_normalize_method = st.radio(
                    "选择标准化方法",
                    ["保持原始点数", "插值到最大点数", "插值到固定点数"],
                    horizontal=True
                )
                
                if energy_normalize_method == "插值到固定点数":
                    fixed_points = st.slider("设置标准化点数", min_value=10, max_value=500, value=100, step=10)
                
                # 为每个任务准备单次能耗数据
                energy_distribution_data = pd.DataFrame()
                
                # 确定目标点数
                if energy_normalize_method == "插值到最大点数":
                    target_points = max([len(task.data) for task in selected_tasks])
                elif energy_normalize_method == "插值到固定点数":
                    target_points = fixed_points
                
                for task in selected_tasks:
                    task_name = task.basic_info['name']
                    # 获取单次能耗数据
                    single_energy = task.data['total_incremental_energy']
                    
                    if energy_normalize_method == "保持原始点数":
                        # 重置索引为问答序号
                        single_energy = single_energy.reset_index(drop=True)
                        # 添加到DataFrame
                        energy_distribution_data[task_name] = single_energy
                    else:
                        # 对单次能耗数据进行插值
                        interpolated_energy = interpolate_data(single_energy, target_points)
                        # 使用统一的索引添加到DataFrame
                        if energy_distribution_data.empty:
                            energy_distribution_data = pd.DataFrame(index=range(target_points))
                        energy_distribution_data[task_name] = interpolated_energy
                
                # 使用折线图展示分布趋势
                st.line_chart(energy_distribution_data)
                
                # 创建分布摘要数据
                distribution_summary = pd.DataFrame({
                    '任务': [],
                    '最小值': [],
                    '25%分位数': [],
                    '中位数': [],
                    '75%分位数': [],
                    '最大值': [],
                    '平均值': [],
                    '标准差': []
                })
                
                for task in selected_tasks:
                    task_name = task.basic_info['name']
                    energy_stats = task.data['total_incremental_energy'].describe()
                    
                    new_row = pd.DataFrame({
                        '任务': [task_name],
                        '最小值': [energy_stats['min']],
                        '25%分位数': [energy_stats['25%']],
                        '中位数': [energy_stats['50%']],
                        '75%分位数': [energy_stats['75%']],
                        '最大值': [energy_stats['max']],
                        '平均值': [energy_stats['mean']],
                        '标准差': [energy_stats['std']]
                    })
                    
                    distribution_summary = pd.concat([distribution_summary, new_row])
                
                # 显示分布统计摘要
                st.markdown("##### 能耗分布统计摘要")
                st.dataframe(distribution_summary)
                
                with st.expander("查看图表说明"):
                    if energy_normalize_method == "保持原始点数":
                        st.markdown("""
                        **单次能耗分布对比说明：**
                        - X轴代表问答序号
                        - Y轴代表单次能耗，单位为Wh
                        - 每条线代表一个选中的任务
                        - 注意：各任务行数不同，直接比较时要考虑这一点
                        - 统计摘要表格提供了各任务能耗分布的关键统计指标
                        """)
                    else:
                        st.markdown("""
                        **单次能耗分布对比说明：**
                        - X轴代表标准化后的问答序号
                        - Y轴代表单次能耗，单位为Wh
                        - 每条线代表一个选中的任务
                        - 通过插值处理，所有任务具有相同的点数，便于直接比较
                        - 统计摘要表格提供了各任务能耗分布的关键统计指标（基于原始数据）
                        """)
                     
            with trend_tabs[2]:
                st.markdown("#### 实时功率变化对比")
                
                # 功率变化的标准化设置
                power_normalize_method = st.radio(
                    "功率对比标准化方法",
                    ["百分比进度", "插值到相同点数"],
                    horizontal=True
                )
                
                if power_normalize_method == "插值到相同点数":
                    power_points = st.slider("功率对比标准化点数", min_value=10, max_value=500, value=100, step=10)
                
                # 计算并绘制实时功率变化
                power_trend_data = pd.DataFrame()
                
                for task in selected_tasks:
                    task_name = task.basic_info['name']
                    # 计算时间间隔（秒）
                    task.data['start_time'] = pd.to_datetime(task.data['start_time'])
                    task.data['end_time'] = pd.to_datetime(task.data['end_time'])
                    task.data['duration'] = (task.data['end_time'] - task.data['start_time']).dt.total_seconds()
                    
                    # 计算实时功率 (W = J/s, 能耗单位为Wh，需要转换为W)
                    # 功率 = 能耗(Wh) * 3600 / 持续时间(s)
                    task.data['power'] = task.data['total_incremental_energy'] * 3600 / task.data['duration']
                    
                    if power_normalize_method == "百分比进度":
                        # 创建百分比进度索引
                        progress_pct = np.linspace(0, 100, len(task.data))
                        # 添加到DataFrame
                        task_df = pd.DataFrame({task_name: task.data['power'].values}, index=progress_pct)
                        power_trend_data = pd.concat([power_trend_data, task_df], axis=1)
                    else:
                        # 对功率数据进行插值
                        interpolated_power = interpolate_data(task.data['power'], power_points)
                        # 使用统一的索引添加到DataFrame
                        if power_trend_data.empty:
                            power_trend_data = pd.DataFrame(index=range(power_points))
                        power_trend_data[task_name] = interpolated_power
                
                # 绘制实时功率对比图
                st.line_chart(power_trend_data)
                
                with st.expander("查看图表说明"):
                    if power_normalize_method == "百分比进度":
                        st.markdown("""
                        **实时功率变化对比说明：**
                        - X轴代表任务进度百分比，从0%到100%
                        - Y轴代表实时功率，单位为W (瓦特)
                        - 每条线代表一个选中的任务
                        - 峰值表示该时刻能耗强度最高
                        """)
                    else:
                        st.markdown("""
                        **实时功率变化对比说明：**
                        - X轴代表标准化后的数据点序号
                        - Y轴代表实时功率，单位为W (瓦特)
                        - 每条线代表一个选中的任务
                        - 通过插值处理，所有任务具有相同的点数，便于直接比较
                        - 峰值表示该时刻能耗强度最高
                        """)

# 重置激活页
st.session_state.current_page_index = 4



import streamlit as st
from energy_tracker_core.Visulization.workspace_task import WorkSpace, Task
import pandas as pd

# 页面配置
st.set_page_config(
    page_title="绘图展板",
    page_icon=":material/palette:",
    layout="wide"
)


st.title("🎨 绘图展板")
ws: WorkSpace = None
ws = st.session_state.get("workspace", None)


def button_callback(task:Task):
    """
    点击按钮后触发回调，设置当前绘图任务
    """

    st.session_state.task_to_plot = task


# 检查前序工作
if 'workspace' not in st.session_state:
    st.warning("请先在 CSV 管理页面初始化工作目录。")
else:
    if 'edited_task_showframe' not in st.session_state:
        st.warning("请先在 CSV 管理页面至少选择一个要分析的csv文件。")
    else:
        # 遍历sw.task_showframe，获取所有选中的任务名称
        selected_task_name = st.session_state.edited_task_showframe.query("is_selected == True")["name"]
        
        # 根据选中的任务名称，获取对应的任务对象,并添加到列表中
        selected_tasks = []
        for task_name in selected_task_name:
            selected_mask = st.session_state.edited_task_showframe['is_selected'] == True
            selected_names = st.session_state.edited_task_showframe[selected_mask]['name'].tolist()
            selected_tasks = [ws.get_task_by_name(name) for name in selected_names]
        
        # 构造侧边栏切换
        st.sidebar.title("选择任务视图")
        with st.sidebar:
            current_index = None
            for task in selected_tasks:
                st.sidebar.button(task.basic_info["name"], 
                                    key=task.basic_info["name"],
                                    on_click=button_callback,
                                    args=(task,))
        
        if "task_to_plot" not in st.session_state:
            st.markdown("### 请在左侧点击要可视化的任务")
        else:
            task:Task
            task = st.session_state.task_to_plot
            task.analyse()
            st.markdown(f"### 任务名称: {task.basic_info['name']}")
            st.markdown(f"#### 任务路径: {task.csv_filepath}")
            # 创建三列布局
            col1, col2, col3 = st.columns(3)
            
            # 第一列显示基本统计信息
            with col1:
                st.markdown("#### 基本统计")
                st.metric("问答总数", f"{task.statistics['问答总数']}次")
                st.metric("正确率", f"{task.statistics['正确率']:.2%}")
                st.metric("平均每次问答时间", f"{task.statistics['平均每次问答时间(秒)']:.2f}秒")

            # 第二列显示能耗统计
            with col2:
                st.markdown("#### 能耗统计 (Wh)")
                st.metric("总能耗", f"{task.statistics['总能耗(Wh)']:.4f}")
                st.metric("CPU总能耗", f"{task.statistics['CPU总能耗(Wh)']:.4f}")
                st.metric("GPU总能耗", f"{task.statistics['GPU总能耗(Wh)']:.4f}")
                

            # 第三列显示功率统计
            with col3:
                st.markdown("#### 功率统计 (W)")
                st.metric("总平均功率", f"{task.statistics['总平均功率(W)']:.4f}")
                st.metric("CPU平均功率", f"{task.statistics['CPU平均功率(W)']:.4f}")
                st.metric("GPU平均功率", f"{task.statistics['GPU平均功率(W)']:.4f}")
                

            # 创建能耗趋势图
            st.markdown("#### 能耗趋势")
            
            # 创建累计能耗趋势线图
            st.markdown("##### 累计能耗趋势")
            trend_chart_data = pd.DataFrame({
                'CPU累计能耗': task.data['cpu_incremental_energy'].cumsum(),
                'GPU累计能耗': task.data['gpu_incremental_energy'].cumsum(), 
                '总累计能耗': task.data['total_incremental_energy'].cumsum()
            })
            st.line_chart(trend_chart_data)

            # 创建每次问答能耗柱状图
            st.markdown("##### 每次问答能耗")
            bar_chart_data = pd.DataFrame({
                'CPU单次能耗': task.data['cpu_incremental_energy'],
                'GPU单次能耗': task.data['gpu_incremental_energy']
            })
            st.bar_chart(bar_chart_data)

            # 显示能耗最高的问答记录
            st.markdown("#### 能耗最高的问答记录")
            high_energy_df = pd.DataFrame(task.statistics['能耗最高问答'])
            st.dataframe(high_energy_df[['question', 'total_incremental_energy', 'duration']])

            # 显示能耗最低的问答记录
            st.markdown("#### 能耗最低的问答记录") 
            low_energy_df = pd.DataFrame(task.statistics['能耗最低问答'])
            st.dataframe(low_energy_df[['question', 'total_incremental_energy', 'duration']])


# 重置激活页
st.session_state.current_page_index = 3


"""
此模块封装了命令行日志输出和交互功能，使用rich库提供丰富的文本显示效果。

console: rich.console.Console 对象，用于输出日志和消息。
logger: rich.logging.Logger 对象，用于记录日志。
StageAdapter: 日志适配器类，用于在日志消息中添加阶段信息。
"""

from .command_line_interface import console, logger, StageAdapter

__all__ = ['console', 'logger', 'StageAdapter']



"""
本模块提供能耗测量和计算以及日志持久化的类。
"""

# 标准库
import datetime
from typing import Generator, Optional
# 自定义库
from energy_tracker_core.HardwareTracker.Tracker import power_monitor
from energy_tracker_core.HardwareTracker.CommandLineInterface import console, logger, StageAdapter
from energy_tracker_core.GlobalConfig import *
from energy_tracker_core.HardwareTracker.Tracker.csv_handler import CSV_Handler
from energy_tracker_core.HardwareTracker.Tracker.power_monitor import CPU_Power_Monitor, GPU_Power_Monitor
class InferenceRecord:
    """
    推理记录类，负责记录单次推理过程中的能耗信息。
    """

    def __init__(self, token: str, cpu_name: str, gpu_name: str):
        self.token = token
        self.cpu_name = cpu_name
        self.gpu_name = gpu_name
    
    def start(self, question, cpu_accumulated_energy_start, gpu_accumulated_energy_start):
        self.start_time = datetime.datetime.now()
        self.question = question
        self.cpu_accumulated_energy_start = cpu_accumulated_energy_start
        self.gpu_accumulated_energy_start = gpu_accumulated_energy_start
        self.total_accumulated_energy_start = cpu_accumulated_energy_start + gpu_accumulated_energy_start
    
    def stop(self, answer, is_correct, is_valid, cpu_accumulated_energy_end, gpu_accumulated_energy_end):
        self.end_time = datetime.datetime.now()
        self.answer = answer
        self.is_correct = is_correct
        self.is_valid = is_valid
        self.cpu_accumulated_energy_end = cpu_accumulated_energy_end
        self.gpu_accumulated_energy_end = gpu_accumulated_energy_end
        self.total_accumulated_energy_end = cpu_accumulated_energy_end + gpu_accumulated_energy_end

    def dump_to_csv(self, csv_handler: CSV_Handler):
        """
        将推理记录写入CSV文件。
        """

        csv_handler.log_ligne(self.start_time, self.end_time, self.cpu_name, self.gpu_name, self.cpu_accumulated_energy_end - self.cpu_accumulated_energy_start, self.gpu_accumulated_energy_end - self.gpu_accumulated_energy_start, self.total_accumulated_energy_end - self.total_accumulated_energy_start, self.question, self.answer, self.is_correct, self.is_valid)

class Tracker:
    """
    能耗测量服务主体，负责异步执行能耗测量任务和响应任务。
    """

    def __init__(self, csv_handler: CSV_Handler=CSV_Handler()):
        self.logger = StageAdapter(logger, {"stage": "Tracker"})
        self.cpu_monitor: Optional[CPU_Power_Monitor] = None
        self.gpu_monitor: Optional[GPU_Power_Monitor] = None
        self.cpu_name = 'Default CPU'   
        self.gpu_name = 'Default GPU'
        if ENABLE_CPU_MONITOR:
            self.cpu_monitor = power_monitor.CPU_Power_Monitor()
            self.cpu_name = self.cpu_monitor.name
        if ENABLE_GPU_MONITOR:
            self.gpu_monitor = power_monitor.GPU_Power_Monitor()
            self.gpu_name = self.gpu_monitor.name

        self.cpu_accumulated_energy: float = 0
        self.gpu_accumulated_energy: float = 0
        self.inference_records_list: dict[str, InferenceRecord] = {}
        self.csv_handler = csv_handler

    def start(self, period:float=5):
        """
        启动能耗测量任务。
        """ 

        self.period = datetime.timedelta(seconds=period)
        self.start_time = datetime.datetime.now()
        self.last_measurement_time = self.start_time
        self.time_generator = self._time_generator()

    def _time_generator(self) -> Generator[datetime.datetime, None, None]:
        expected_next_time = datetime.datetime.now() + self.period
        while True:
            yield expected_next_time
            expected_next_time += self.period

    def do_measurements(self):
        """
        执行一次能耗测量。
        """

        period = datetime.datetime.now() - self.last_measurement_time
        self.last_measurement_time = datetime.datetime.now()
        period = period.total_seconds()
        if self.cpu_monitor is not None:
            append_cpu_energy = self.cpu_monitor.mesure_power()*period/3600
            self.cpu_accumulated_energy += append_cpu_energy
            self.logger.debug(f"CPU能耗追加{append_cpu_energy}Wh")
        if self.gpu_monitor is not None:
            append_gpu_energy = self.gpu_monitor.mesure_power()*period/3600
            self.gpu_accumulated_energy += append_gpu_energy
            self.logger.debug(f"GPU能耗追加{append_gpu_energy}Wh")

    def start_inference_record(self, token: str, question: str):
        """
        开始一次推理记录。
        """
        self.inference_records_list[token] = InferenceRecord(token, self.cpu_name, self.gpu_name)
        self.inference_records_list[token].start(question, self.cpu_accumulated_energy, self.gpu_accumulated_energy)

    def stop_inference_record(self, token: str, answer: str, is_correct: bool, is_valid: bool):
        """
        结束一次推理记录。
        """
        self.inference_records_list[token].stop(answer, is_correct, is_valid, self.cpu_accumulated_energy, self.gpu_accumulated_energy)
        self.inference_records_list[token].dump_to_csv(self.csv_handler)
        del self.inference_records_list[token]

    def stop(self):
        """
        资源回收。
        """
        self.csv_handler.close()
            

 


import streamlit as st


st.set_page_config(
    page_title="欢迎/帮助",
    page_icon="🏠",
    layout="centered"
)
st.title("🎉 欢迎使用CSV Visulization! 🏠")

st.markdown(
    """
    ### 本项目为Energy Tracker的子项目, 用于可视化能耗追踪服务日志
    ### 使用说明
    - 请使用左侧侧边栏切换至对应功能页面。
    - 在各功能页面，根据提示输入或选择参数，页面将自动渲染结果。
    - 目前支持csv日志管理, 单个日志可视化和多日志对比。

    ### 快捷操作
    - Windows: `Ctrl+R` 重载页面
    - macOS: `⌘+R` 重载页面

    ### 联系方式
    有任何问题，请联系：philippe.qu@outlook.com
    """
)

st.info("当前为欢迎/帮助页面，可在侧边栏选择其他页面。")
st.session_state.current_page_index = 1


"""
本模块用于提供检测物理机cpu和gpu功率的类。
"""

# 标准库
import logging
import time
from abc import ABC, abstractmethod
import sys
# 非标准库
import pynvml
import wmi
# 自定义库
from energy_tracker_core.HardwareTracker.CommandLineInterface import console, logger, StageAdapter
from energy_tracker_core.GlobalConfig import *
class BasePowerMonitor(ABC):
    """
    基础监测器类，提供监测器的基本功能。
    """

    @abstractmethod
    def mesure_power(self) -> float:
        """
        执行一次测量，获取当前功率。
        """
    
    # 测试测量功率所需时间，避免测量周期小于测量耗时，导致数据周期紊乱
    @abstractmethod
    def test_measurement_time(self) -> float:
        """
        测试测量功率所需时间。
        """

        pass

class CPU_Power_Monitor(BasePowerMonitor):
    """
    CPU 监测器类，继承自基础监测器类。
    根据OpenHardwareMonitorAPI获取CPU的当前功率和累计能耗。
    """
    def __init__(self):
        self.name: str = wmi.WMI().Win32_Processor()[0].Name
        self.cpu_logger = StageAdapter(logger, {"stage": "CPU monitor"})
        self.last_valid_power = 0.0

        if USE_ACTUAL_CPU_POWER:
            self.initialize()
            self.cpu_logger.info(f"CPU能耗测量启用，将使用LibreHardwareMonitor获取实际功耗, 返回：{CPU_POWER_BIAS} + {CPU_POWER_COEFFICIENT} * 测得功率")
        else:
            self.cpu_logger.warning(f"CPU实际功耗测量被禁用，将返回设置值：{CPU_POWER_BIAS}")

    def initialize(self):
        """
        初始化监测器。
        """

        self.name: str = wmi.WMI().Win32_Processor()[0].Name
        self.wmi_obj = wmi.WMI(namespace="root\\LibreHardwareMonitor")
        # 确认能否通过wmi访问LibreHardwareMonitor的cpu数据。同时传感器名称
        # 可能的功率传感器访问字段

        sensor_name_list = ["CPU Package", "Package"]
        self.sensor_filed = None
        try:
            for sensor_name in sensor_name_list:
                query = f"SELECT Value FROM Sensor WHERE Name = '{sensor_name}' AND SensorType = 'Power'"
                sensor = self.wmi_obj.query(query)
                if not sensor:
                    continue
                self.sensor_filed = sensor_name
                self.is_exist = True
                self.cpu_logger.info(f"CPU监测器连接底层API成功，CPU名称为：{self.name}")
                time = self.test_measurement_time()
                self.cpu_logger.info(f"CPU监测器测量耗时：{time}秒")
                return
            raise Exception("CPU功率传感器未找到")
        except:
            self.cpu_logger.error("CPU_Monitor初始化失败，请确认LibreHardwareMonitor或WMI是否正常工作。")
            sys.exit(1)

    

    def mesure_power(self) -> float:
        """
        执行一次测量，获取CPU的当前功率（单位：瓦特）。
        """
        
        query = f"SELECT Value FROM Sensor WHERE Name = '{self.sensor_filed}' AND SensorType = 'Power'"
        
        try:
            sensor = self.wmi_obj.query(query)
            cpu_power_sensor = sensor[0]
            self.last_valid_power = cpu_power_sensor.Value
            self.cpu_logger.debug(f"GPU功耗测量值：{self.last_valid_power}W")
            return self.last_valid_power*CPU_POWER_COEFFICIENT + CPU_POWER_BIAS
        except:
            self.cpu_logger.error("测量失败，CPU监测器未找到,请确认LibreHardwareMonitor是否正常工作, 或检查功率传感器字段是否对应。")
            sys.exit(1)


    def test_measurement_time(self)->float:
        """
        测试测量功率所需时间。
        """

        start_time = time.time()
        self.mesure_power()
        end_time = time.time()
        return end_time - start_time
        
class GPU_Power_Monitor(BasePowerMonitor):
    """
    GPU 监测器类，继承自基础监测器类。
    根据nvml获取GPU的当前功率和累计能耗。
    """

    def __init__(self):
        self.name = "GPU Name"
        self.last_valid_power = 0.0
        self.gpu_logger = StageAdapter(logger, {"stage": "GPU monitor"})

        if USE_ACTUAL_GPU_POWER:
            self.initialize()
            self.gpu_logger.info(f"GPU实际功耗测量启用, 将使用nvml获取实际功耗, 将返回：{GPU_POWER_BIAS} + {GPU_POWER_COEFFICIENT} * 测得功率")
        else:
            self.gpu_logger.warning(f"GPU实际功耗测量被禁用，将返回设置值：{GPU_POWER_BIAS}")
        
        

    def initialize(self):
        """
        接通底层硬件api
        """

        # 测量失败次数
        self.error_count = 0
        # 测量失败容错次数阈值
        self.error_tolerance = 5
        try:
            pynvml.nvmlInit()
            self.name = pynvml.nvmlDeviceGetName(pynvml.nvmlDeviceGetHandleByIndex(0))
            self.handle = pynvml.nvmlDeviceGetHandleByIndex(0)
            self.gpu_logger.info(f"GPU监测器连接底层API成功，GPU名称为：{self.name}")
            time = self.test_measurement_time()
            self.gpu_logger.info(f"GPU监测器测量耗时：{time}秒")
        except:
            self.gpu_logger.error("GPU_Monitor底层API初始化失败，不能正常访问Nvidia GPU，服务失败")
            sys.exit(1)

    def mesure_power(self):
        try:
            power = pynvml.nvmlDeviceGetPowerUsage(self.handle)
            self.last_valid_power = power / 1000.0  # 转换为瓦特并保存最后有效值
            self.gpu_logger.debug(f"GPU功耗测量值：{self.last_valid_power}W")
            if self.error_count > 0:
                self.gpu_logger.warning(f"GPU进行了一次有效测量，重置错误次数")
                self.error_count = 0
            return self.last_valid_power*GPU_POWER_COEFFICIENT + GPU_POWER_BIAS
        
        except pynvml.NVMLError as e:
            self.error_count += 1
            if self.error_count >= self.error_tolerance:
                self.gpu_logger.error(f"GPU功耗测量错误次数超过阈值，测量终止。")
                sys.exit(1)
            self.gpu_logger.warning(f"GPU功耗测量失败，但没有超过阈值，返回上次有效值")
            return getattr(self, 'last_valid_power', 0.0)*GPU_POWER_COEFFICIENT + GPU_POWER_BIAS

    def test_measurement_time(self)->float:
        """
        测试测量功率所需时间。
        """

        start_time = time.time()
        self.mesure_power()
        end_time = time.time()
        return end_time - start_time

if __name__ == "__main__":
    # 测试本模块能否正常测量能耗

    cpu_monitor = CPU_Power_Monitor()
    gpu_monitor = GPU_Power_Monitor()
    
    for i in range(10):
        print(f"CPU power: {cpu_monitor.mesure_power()}")
        print(f"GPU power: {gpu_monitor.mesure_power()}")
        time.sleep(1)



"""
本模块提供能耗日志记录功能。
"""

# 标准库
import csv
import datetime
import os
from energy_tracker_core.GlobalConfig import *
from energy_tracker_core.HardwareTracker.CommandLineInterface import logger, console, StageAdapter

class CSV_Handler:
    """
    提供一个持久化CSV日志记录器，用于记录能耗数据。
    """

    def __init__(self, filename: str=datetime.datetime.now().strftime("%Y%m%d_%H%M%S")):
        """
        初始化日志记录器，并保持文件对象和 writer 对象长期有效
        """

        question_answer_list = []
        self.filename = filename
        # 固定列
        self.fixed_columns = [
            'start_time',
            'end_time',
            'cpu_name',
            'gpu_name',
            'cpu_incremental_energy',
            'gpu_incremental_energy',
            'total_incremental_energy',
            'question',
            'answer',
            'is_correct',
            'is_valid'
        ]

        csv_file_path = os.path.join(DATA_DIR, self.filename + '.csv')
        self.file = open(csv_file_path, mode='w', newline='', encoding='utf-8')
        self.writer = csv.DictWriter(self.file, fieldnames=self.fixed_columns)
        self.writer.writeheader()
        self.file.flush()  # 立即写入表头

    def log_ligne(self, start_time: datetime.datetime, end_time: datetime.datetime, cpu_name: str, gpu_name: str, cpu_incremental_energy: float,
            gpu_incremental_energy: float, total_incremental_energy: float, question: str = 'NULL', answer: str = 'NULL', is_correct: bool = False, is_valid: bool = False):
        """
        写入一条日志记录


        """
        # 构造一行日志记录
        row = {
            'start_time': start_time.strftime("%Y-%m-%d %H:%M:%S"),
            'end_time': end_time.strftime("%Y-%m-%d %H:%M:%S"),
            'cpu_name': cpu_name,
            'gpu_name': gpu_name,
            'cpu_incremental_energy': cpu_incremental_energy,
            'gpu_incremental_energy': gpu_incremental_energy,
            'total_incremental_energy': total_incremental_energy,
            'question': question,
            'answer': answer,
            'is_correct': is_correct,
            'is_valid': is_valid
        }

        self.writer.writerow(row)
        self.file.flush()  # 及时刷新写入的数据

    def close(self):
        """关闭文件对象，结束日志记录"""
        self.file.close()




import click
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TimeElapsedColumn
from rich.console import Console
import time

@click.group()
def cli():
    """能源追踪器命令行工具

    这个工具可以用来启动异步服务和可视化界面。
    """
    pass

@cli.command('server')
@click.option('--host', default='127.0.0.1', help='服务器主机地址')
@click.option('--port', default=8000, type=int, help='服务器端口')
def start_server(host, port):
    """启动硬件监控异步服务。

    这个服务会收集CPU和GPU的能耗数据。
    """
    from energy_tracker_core.HardwareTracker.AsyncService.async_server import app
    from aiohttp import web
    
    console = Console()
    click.echo(f"正在启动异步服务器，地址：{host}，端口：{port}")
    
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),
        TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
        TimeElapsedColumn(),
        console=console,
    ) as progress:
        task = progress.add_task("[cyan]服务器启动中...", total=100)
        
        # 模拟启动过程
        for i in range(100):
            time.sleep(0.02)  # 添加小延迟使进度条可见
            progress.update(task, advance=1)
            
            # 在特定进度点显示额外信息
            if i == 30:
                progress.print("[yellow]正在初始化硬件监控...")
            elif i == 60:
                progress.print("[yellow]正在配置数据收集器...")
            elif i == 90:
                progress.print("[yellow]正在启动异步服务...")
    
    console.print("[green]服务器启动完成！[/green]")
    web.run_app(app, host=host, port=port)

@cli.command('viz')
def start_visualization():
    """启动Streamlit可视化界面。

    这个界面用于展示能耗数据的图表和统计信息。
    """
    import streamlit.web.cli as stcli
    import sys
    from pathlib import Path
    
    click.echo("正在启动可视化界面...")
    
    # 获取app.py的路径
    app_path = Path(__file__).parent / "Visulization" / "app.py"
    
    # 使用streamlit CLI来运行应用
    sys.argv = ["streamlit", "run", str(app_path), "--server.headless", "true"]
    stcli.main()

@cli.command('info')
def system_info():
    """显示系统硬件信息。

    包括CPU、GPU等硬件的详细信息和当前状态。
    """
    click.echo("正在收集系统硬件信息...")
    try:
        import platform
        import wmi
        import pynvml
        
        # 初始化WMI和NVML
        w = wmi.WMI()
        pynvml.nvmlInit()
        
        # 获取系统信息
        system_name = platform.system()
        system_version = platform.version()
        
        # 获取CPU信息
        cpu_info = w.Win32_Processor()[0]
        cpu_name = cpu_info.Name
        cpu_cores = cpu_info.NumberOfCores
        
        # 获取GPU信息
        device_count = pynvml.nvmlDeviceGetCount()
        gpu_info = []
        
        for i in range(device_count):
            handle = pynvml.nvmlDeviceGetHandleByIndex(i)
            name = pynvml.nvmlDeviceGetName(handle)
            gpu_info.append(name)
        
        # 输出信息
        click.echo(f"系统: {system_name} {system_version}")
        click.echo(f"CPU: {cpu_name} ({cpu_cores}核)")
        click.echo("GPU:")
        for i, gpu in enumerate(gpu_info):
            click.echo(f"  [{i}] {gpu}")
            
        pynvml.nvmlShutdown()
    except Exception as e:
        click.echo(f"获取系统信息失败: {str(e)}")
        click.echo("请确保安装了所有必要的依赖并且有适当的权限。")

if __name__ == '__main__':
    cli()



from aiohttp import web
import asyncio
import datetime
import logging

from energy_tracker_core.HardwareTracker.CommandLineInterface import console, logger, StageAdapter
from energy_tracker_core.HardwareTracker.Tracker.tracker import Tracker


# 设置asyncio的日志级别为WARNING，只显示错误信息，避免日志格式混乱
logging.getLogger('asyncio').setLevel(logging.ERROR)

# 设置aiohttp的日志级别为ERROR，只显示错误信息，避免日志格式混乱
logging.getLogger('aiohttp').setLevel(logging.ERROR)
logging.getLogger('aiohttp.access').setLevel(logging.ERROR)
logging.getLogger('aiohttp.client').setLevel(logging.ERROR)
logging.getLogger('aiohttp.internal').setLevel(logging.ERROR)
logging.getLogger('aiohttp.server').setLevel(logging.ERROR)
logging.getLogger('aiohttp.web').setLevel(logging.ERROR)
logging.getLogger('aiohttp.websocket').setLevel(logging.ERROR)

# 创建不同阶段的日志适配器
server_log = StageAdapter(logger, {"stage": "服务信息"})
request_log = StageAdapter(logger, {"stage": "请求处理"})
auto_mesure_log = StageAdapter(logger, {"stage": "自动能耗测量"})

# 创建日志中间件
@web.middleware
async def log_middleware(request, handler):
    # 记录请求开始时间
    start_time = datetime.datetime.now()
    
    # 记录请求信息，包括header
    headers_info = {k: v for k, v in request.headers.items()}
    request_log.debug(f"收到请求: {request.method} {request.path} {headers_info}")
    
    try:
        # 处理请求
        response = await handler(request)
        
        # 记录响应信息
        process_time = (datetime.datetime.now() - start_time).total_seconds()
        request_log.debug(f"请求处理完成: {request.method} {request.path} - 状态码: {response.status} - 处理时间: {process_time:.3f}秒")
        
        return response
    except Exception as e:
        # 记录错误信息
        request_log.error(f"请求处理出错: {request.method} {request.path} - 错误: {str(e)}")
        raise

async def hd_post_start_inference(request: web.Request):
    # 读取Token
    token = request.headers.get('Token', '')
    request_log.info(f"能耗追踪任务开始，Token: {token}")
    data = await request.post()
    tracker.start_inference_record(token, str(data['question']))
    return web.json_response({"status": "success"})

async def hd_post_stop_inference(request: web.Request):
    # 读取Token
    token = request.headers.get('Token', '')
    request_log.info(f"能耗追踪任务结束，Token: {token}")
    
    data = await request.post()       
    tracker.stop_inference_record(token, str(data['answer']), bool(data['is_correct']), bool(data['is_valid']))
    return web.json_response({"status": "success"})

async def periodic_task():
    while True:
        tracker.do_measurements()
        waited_time = (next(tracker.time_generator) - datetime.datetime.now()).total_seconds()
        while waited_time < 0:
            waited_time = (next(tracker.time_generator) - datetime.datetime.now()).total_seconds()
        await asyncio.sleep(waited_time)

async def start_background_tasks(app: web.Application):
    # 创建后台任务
    app['periodic_task'] = asyncio.create_task(periodic_task())
    server_log.info("后台能耗测量任务已启动")

async def cleanup_background_tasks(app: web.Application):
    # 清理后台任务
    app['periodic_task'].cancel()
    await app['periodic_task']
    server_log.info("后台能耗测量任务已清理")


# 创建能耗追踪器
tracker = Tracker()
tracker.start()

# 创建应用
app = web.Application()

# 添加中间件
app.middlewares.append(log_middleware)

# 添加路由
app.add_routes([web.post('/start_inference', hd_post_start_inference),
                web.post('/stop_inference', hd_post_stop_inference)])

# 添加启动和清理后台任务的处理函数
app.on_startup.append(start_background_tasks)
app.on_cleanup.append(cleanup_background_tasks)


if __name__ == '__main__':
    server_log.info("异步服务器启动中...")
    web.run_app(app, host='127.0.0.1', port=8000)
    



"""
此模块负责加载配置文件、定义全局路径常量，以及提供对配置参数的访问。

PROJECT_ROOT: 项目根目录路径。
CONFIG_DIR: 配置文件目录路径。
RESOURCES_DIR: 资源文件目录路径。
DATA_DIR: 数据文件目录路径。
ENABLE_CPU_MONITOR: 是否启用CPU监控。
ENABLE_GPU_MONITOR: 是否启用GPU监控。
USE_ACTUAL_CPU_POWER: 是否使用实际CPU功率。
USE_ACTUAL_GPU_POWER: 是否使用实际GPU功率。
MEASUREMENT_INTERVAL: 测量间隔时间。
CPU_POWER_BIAS: CPU功率偏移量。
CPU_POWER_COEFFICIENT: CPU功率系数。
GPU_POWER_BIAS: GPU功率偏移量。
GPU_POWER_COEFFICIENT: GPU功率系数。
"""

from .globals import (
    # 路径
    PACKAGE_ROOT, 
    DATA_DIR,
    CONFIG_DIR,
    # 能源追踪配置
    ENABLE_CPU_MONITOR,
    ENABLE_GPU_MONITOR,
    USE_ACTUAL_CPU_POWER,
    USE_ACTUAL_GPU_POWER,
    MEASUREMENT_INTERVAL,
    CPU_POWER_BIAS,
    CPU_POWER_COEFFICIENT,
    GPU_POWER_BIAS,
    GPU_POWER_COEFFICIENT
)

__all__ = [
    # 路径
    'PACKAGE_ROOT', 
    'CONFIG_DIR', 
    'DATA_DIR',
    # 能源追踪配置
    'ENABLE_CPU_MONITOR',
    'ENABLE_GPU_MONITOR',
    'USE_ACTUAL_CPU_POWER',
    'USE_ACTUAL_GPU_POWER',
    'MEASUREMENT_INTERVAL',
    'CPU_POWER_BIAS',
    'CPU_POWER_COEFFICIENT',
    'GPU_POWER_BIAS',
    'GPU_POWER_COEFFICIENT'
]



import streamlit as st
# 定义多页面导航

welcome_page = st.Page(
    "pages/1_welcome.py",  # 主入口脚本
    title="欢迎",
    icon=":material/home:"
)
csv_page = st.Page(
    "pages/2_CSV管理.py",  # CSV 管理脚本
    title="CSV 管理",
    icon=":material/folder:"
)
drawing_page = st.Page(
    "pages/3_绘图展板.py",  # 绘图展板脚本
    title="绘图展板",
    icon=":material/palette:"
)
compare_page = st.Page(
    "pages/4_结果对比.py",  # 结果对比脚本
    title="结果对比",
    icon=":material/compare:"
)
pages = [welcome_page, csv_page, drawing_page, compare_page]
pg = st.navigation(pages)



pg.run()



from __future__ import annotations
import os
from pathlib import Path
from dataclasses import dataclass, field
import pandas as pd
import numpy as np

@dataclass
class Task:
    """
    处理单个 CSV 文件的任务。
    属性:
        csv_filepath: str        # 初始化时必须提供
        basic_info: dict         # 扫描得到的基本信息字典，包括开始/结束时间、时长、行数和首条问题
        data: pd.DataFrame       # 加载后的 DataFrame（init=False，运行时赋值）
        statistics: dict         # 统计分析结果字典（init=False，运行时赋值）
    方法:
        scan, load, analyse
    """

    csv_filepath: str
    basic_info: dict = field(default_factory=dict)
    data: pd.DataFrame = field(default=None, init=False)
    statistics: dict = field(default_factory=dict, init=False)

    def __post_init__(self) -> None:
        # 确保路径为绝对路径
        self.csv_filepath = str(Path(self.csv_filepath).absolute())

    def scan(self) -> None:
        """扫描 CSV 文件的核心信息：开始时间、结束时间、时长、行数和首条问题"""
        df = pd.read_csv(self.csv_filepath)
        if df.empty:
            raise ValueError(f"CSV 文件 {self.csv_filepath} 为空，无法扫描基本信息。")
        start = pd.to_datetime(df.iloc[0]['start_time'])
        end = pd.to_datetime(df.iloc[-1]['end_time'])
        duration = end - start
        capacity = len(df)
        first_question = df.iloc[0].get('question', 'None')
        self.basic_info = {
            'name': os.path.basename(self.csv_filepath),
            'start_time': start.strftime('%Y-%m-%d %H:%M:%S'),
            'end_time': end.strftime('%Y-%m-%d %H:%M:%S'),
            'duration': str(duration),
            'capacity': int(capacity),
            'first_question': first_question
        }

    def load(self) -> None:
        """将 CSV 文件加载为 DataFrame"""
        self.data = pd.read_csv(self.csv_filepath)

    def analyse(self, top_n: int = 3) -> None:
        """对 DataFrame 执行所需统计分析，包括累计能耗、功率计算和极值统计"""
        if self.data is None or self.data.empty:
            self.load()
        
        df = self.data.copy()
        total_count = int(df.shape[0])
        
        # 1. 计算每行的持续时间并累加
        df['start_time'] = pd.to_datetime(df['start_time'])
        df['end_time'] = pd.to_datetime(df['end_time'])
        df['duration'] = (df['end_time'] - df['start_time']).dt.total_seconds()
        total_duration_sec = float(df['duration'].sum())
        
        # 2. 累加能耗值
        total_cpu_energy = float(df['cpu_incremental_energy'].sum())
        total_gpu_energy = float(df['gpu_incremental_energy'].sum())
        total_energy = float(df['total_incremental_energy'].sum())
        
        # 3. 计算平均功率 (能量/时间)
        avg_cpu_power = float(total_cpu_energy*3600 / total_duration_sec) if total_duration_sec else 0.0
        avg_gpu_power = float(total_gpu_energy*3600 / total_duration_sec) if total_duration_sec else 0.0
        avg_total_power = float(total_energy*3600 / total_duration_sec) if total_duration_sec else 0.0
        
        # 平均每次问答的能耗
        avg_cpu_energy_per_q = float(total_cpu_energy / total_count) if total_count else 0.0
        avg_gpu_energy_per_q = float(total_gpu_energy / total_count) if total_count else 0.0
        avg_energy_per_q = float(total_energy / total_count) if total_count else 0.0
        
        # 4. 找到最大和最小值对应的行
        max_duration_row = df.loc[[df['duration'].idxmax()]]
        min_duration_row = df.loc[[df['duration'].idxmin()]]
        
        max_cpu_energy_row = df.loc[[df['cpu_incremental_energy'].idxmax()]]
        min_cpu_energy_row = df.loc[[df['cpu_incremental_energy'].idxmin()]]
        
        max_gpu_energy_row = df.loc[[df['gpu_incremental_energy'].idxmax()]]
        min_gpu_energy_row = df.loc[[df['gpu_incremental_energy'].idxmin()]]
        
        max_total_energy_row = df.loc[[df['total_incremental_energy'].idxmax()]]
        min_total_energy_row = df.loc[[df['total_incremental_energy'].idxmin()]]
        
        # 将最大/最小值行转换为numpy数组形式备用
        max_duration_array = np.array([max_duration_row['start_time'].iloc[0], max_duration_row['end_time'].iloc[0], 
                                      max_duration_row['duration'].iloc[0], max_duration_row['total_incremental_energy'].iloc[0]])
        min_duration_array = np.array([min_duration_row['start_time'].iloc[0], min_duration_row['end_time'].iloc[0], 
                                      min_duration_row['duration'].iloc[0], min_duration_row['total_incremental_energy'].iloc[0]])
        
        max_cpu_energy_array = np.array([max_cpu_energy_row['start_time'].iloc[0], max_cpu_energy_row['end_time'].iloc[0], 
                                        max_cpu_energy_row['cpu_incremental_energy'].iloc[0]])
        min_cpu_energy_array = np.array([min_cpu_energy_row['start_time'].iloc[0], min_cpu_energy_row['end_time'].iloc[0], 
                                        min_cpu_energy_row['cpu_incremental_energy'].iloc[0]])
        
        max_gpu_energy_array = np.array([max_gpu_energy_row['start_time'].iloc[0], max_gpu_energy_row['end_time'].iloc[0], 
                                        max_gpu_energy_row['gpu_incremental_energy'].iloc[0]])
        min_gpu_energy_array = np.array([min_gpu_energy_row['start_time'].iloc[0], min_gpu_energy_row['end_time'].iloc[0], 
                                        min_gpu_energy_row['gpu_incremental_energy'].iloc[0]])
        
        max_total_energy_array = np.array([max_total_energy_row['start_time'].iloc[0], max_total_energy_row['end_time'].iloc[0], 
                                          max_total_energy_row['total_incremental_energy'].iloc[0]])
        min_total_energy_array = np.array([min_total_energy_row['start_time'].iloc[0], min_total_energy_row['end_time'].iloc[0], 
                                          min_total_energy_row['total_incremental_energy'].iloc[0]])
        
        # 能耗最高/最低的 N 次问答
        highest_energy_qas = df.nlargest(top_n, 'total_incremental_energy').to_dict(orient='records')
        lowest_energy_qas = df.nsmallest(top_n, 'total_incremental_energy').to_dict(orient='records')
        
        # 正确率
        correct_rate = float(df['is_correct'].sum() / total_count) if total_count else 0.0
        # 更新统计信息
        self.statistics = {
            '问答总数': total_count,
            '总持续时间(秒)': total_duration_sec,
            '正确率': correct_rate,
            '平均每次问答时间(秒)': float(total_duration_sec / total_count) if total_count else 0.0,
            
            # 累计能耗数据 (Wh)
            'CPU总能耗(Wh)': total_cpu_energy,
            'GPU总能耗(Wh)': total_gpu_energy,
            '总能耗(Wh)': total_energy,
            
            # 平均每次问答能耗 (Wh)
            'CPU平均每次问答能耗(Wh)': avg_cpu_energy_per_q,
            'GPU平均每次问答能耗(Wh)': avg_gpu_energy_per_q,
            '平均每次问答总能耗(Wh)': avg_energy_per_q,
            
            # 功率数据 (W)
            'CPU平均功率(W)': avg_cpu_power,
            'GPU平均功率(W)': avg_gpu_power,
            '总平均功率(W)': avg_total_power,
            
            # 最大/最小值数组
            '最长持续时间数组': max_duration_array,
            '最短持续时间数组': min_duration_array,
            'CPU最大能耗数组': max_cpu_energy_array,
            'CPU最小能耗数组': min_cpu_energy_array,
            'GPU最大能耗数组': max_gpu_energy_array,
            'GPU最小能耗数组': min_gpu_energy_array,
            '最大总能耗数组': max_total_energy_array,
            '最小总能耗数组': min_total_energy_array,
            
            # 最值DataFrame
            '最长持续时间数据': max_duration_row,
            '最短持续时间数据': min_duration_row,
            'CPU最大能耗数据': max_cpu_energy_row,
            'CPU最小能耗数据': min_cpu_energy_row,
            'GPU最大能耗数据': max_gpu_energy_row,
            'GPU最小能耗数据': min_gpu_energy_row,
            '最大总能耗数据': max_total_energy_row,
            '最小总能耗数据': min_total_energy_row,
            
            # 极值记录
            '能耗最高问答': highest_energy_qas,
            '能耗最低问答': lowest_energy_qas
        }

@dataclass
class WorkSpace:
    """
    管理工作目录下的 CSV 文件，并生成对应 Task 列表。
    属性:
        workspace_dirpath: str       # 初始化时必须提供
        csvfile_list: list[str]      # 扫描时赋值（init=False）
        task_list: list[Task]        # scan 时初始化并赋值（init=False）
    方法:
        scan, analyse
    """
    workspace_dirpath: str
    csvfile_list: list[str] = field(default_factory=list, init=False)
    task_list: list[Task] = field(default_factory=list, init=False)
    task_showframe: pd.dataframe = field(default=None, init=False)

    def __post_init__(self) -> None:
        self.workspace_dirpath = str(Path(self.workspace_dirpath).absolute())

    def scan(self) -> None:
        """扫描目录下所有 CSV 文件，初始化 Task 并执行 scan"""
        if not os.path.isdir(self.workspace_dirpath):
            raise NotADirectoryError(f"{self.workspace_dirpath} 不是有效目录")
        self.csvfile_list = [f for f in os.listdir(self.workspace_dirpath) if f.lower().endswith('.csv')]
        self.task_list = []
        for fname in self.csvfile_list:
            path = os.path.join(self.workspace_dirpath, fname)
            task = Task(path)
            task.scan()
            self.task_list.append(task)
        # 生成任务描述 DataFrame
        self.task_showframe = pd.DataFrame([task.basic_info for task in self.task_list])
        self.task_showframe['is_selected'] = False

    def analyse(self, top_n: int = 3) -> None:
        """对所有 Task 执行 load 与 analyse"""
        if not self.task_list:
            self.scan()
        for task in self.task_list:
            task.load()
            task.analyse(top_n=top_n)

    def get_task_by_name(self, name: str) -> Task:
        """根据任务名称获取对应的 Task 对象"""
        for task in self.task_list:
            if task.basic_info['name'] == name:
                return task
        raise ValueError(f"未找到名称为 {name} 的任务")
    
# 示例用法
if __name__ == "__main__":
    ws = WorkSpace("D:\\3PI_2024\\EnergyTracker_Dev\\log")
    ws.scan()
    for t in ws.task_list:
        print(f"Basic info for {t.csv_filepath}: {t.basic_info}")
    ws.analyse(top_n=0)
    for t in ws.task_list:
        print(f"Statistics for {t.csv_filepath}: {t.statistics}")



import streamlit as st
from pathlib import Path
from energy_tracker_core.Visulization.workspace_task import WorkSpace
import time
import pandas as pd

# 页面配置
st.set_page_config(
    page_title="CSV 文件管理",
    page_icon="📂",
    layout="wide"
)


st.title("📂 CSV 文件管理")
st.markdown("### 工作目录")


# 输入并确认 workspace 目录
col1, col2, col3 = st.columns([12, 1, 1], vertical_alignment="bottom")
with col1:
    dir_input = st.text_input(
        "请输入包含 CSV 文件的工作目录绝对路径", value=st.session_state.get("workspace_dirpath", ""),
        help="日志文件默认放在项目根目录的log文件夹下",
        label_visibility='visible'
    )
with col2:
    confirm_click = st.button("确认", help='确认新路径',key="confirm_dir",use_container_width=True)

with col3:
    update_click = st.button("更新", help='重新扫描当前路径',key="rescan_dir",use_container_width=True)

if confirm_click:
    try:
        ws = WorkSpace(dir_input)
        ws.scan()
        st.session_state.workspace = ws
        st.success("Workspace 初始化成功，已扫描目录中的 CSV 文件。")
    except Exception as e:
        st.error(f"初始化失败：{e}")
if update_click:
    if "workspace" in st.session_state:
        ws = st.session_state.workspace
        try:
            ws.scan()
            st.success("Workspace 更新成功，已重新扫描目录中的 CSV 文件。")
        except Exception as e:
            st.error(f"更新失败：{e}")
    else:
        st.warning("请先初始化 Workspace。")
    
# 如果已初始化 workspace，则展示文件列表和基本信息
if "workspace" in st.session_state:
    if st.session_state.current_page_index != 2:
        st.session_state.current_page_index = 2
        st.session_state.workspace.task_showframe = st.session_state.edited_task_showframe.copy()
        st.rerun()
    with st.form(key="file_selection_form"):
        st.session_state.edited_task_showframe = st.data_editor(st.session_state.workspace.task_showframe)
        is_submitted = st.form_submit_button(label="提交",use_container_width=True)
    if is_submitted:
        st.success("已提交选择")
    
    
# 重置激活页
st.session_state.current_page_index = 2


from pathlib import Path
from pydantic import BaseModel, Field
from typing import Optional

class EnergyTrackerConfig(BaseModel):
    
    enable_cpu_power_monitor: bool = True
    use_actual_cpu_power: bool = True
    enable_gpu_power_monitor: bool = True
    use_actual_gpu_power: bool = True
    measurement_interval: float = 3.0
    cpu_power_bias: float = 0.0
    cpu_power_coefficient: float = 1.0
    gpu_power_bias: float = 0.0
    gpu_power_coefficient: float = 1.0



class GlobalConfig(BaseModel):
    energy_tracker_config: EnergyTrackerConfig

def load_config(path: Optional[Path | str] = None) -> GlobalConfig:
    # 自动定位项目根下的 config/config.json
    if path == None:
        # 从当前目录开始向上搜索，直到找到同级目录中有config目录
        current_path = Path(__file__).resolve().parent
        while current_path.parent != current_path:  # 防止到达根目录还未找到
            # 检查当前目录同级是否有config目录
            config_dir = current_path.parent / "config"
            if config_dir.exists() and config_dir.is_dir():
                break
            current_path = current_path.parent
        
        if current_path.parent == current_path:  # 到达根目录仍未找到
            raise FileNotFoundError("无法找到包含config目录的项目根目录")
            
        project_root = current_path.parent
        print(f"找到项目根目录: {project_root}")
        path = project_root / "config" / "config.json"
    with open(path) as f:
        return GlobalConfig.model_validate_json(f.read())

class PathSettings:
    def __init__(self):
        # 项目根目录
        # 自下而上搜索同级目录中带有config文件夹的父目录作为项目包目录
        current_path = Path(__file__).resolve().parent
        while current_path.parent != current_path:  # 防止到达根目录还未找到
            # 检查当前目录同级是否有config目录
            config_dir = current_path.parent / "config"
            if config_dir.exists() and config_dir.is_dir():
                self.package_root = current_path.parent
                break
            current_path = current_path.parent
        
        if current_path.parent == current_path:  # 到达根目录仍未找到
            # 如果找不到，回退到默认值
            self.package_root = Path(__file__).resolve().parents[3]

        # 各功能目录，默认基于 project_root 生成
        self.data_dir = self.package_root / "data"
        self.config_dir = self.package_root / "config"

# 全局单例，其他模块直接 `from your_package.paths import PATHS` 使用
PATHS = PathSettings()

# 测试用例
if __name__ == "__main__":
    # 测试配置文件加载
    config = load_config()
    print("能源追踪配置:")
    print(f"  启用CPU监控: {config.energy_tracker_config.enable_cpu_power_monitor}")
    print(f"  启用GPU监控: {config.energy_tracker_config.enable_gpu_power_monitor}")
    
    # 测试路径设置
    paths = PathSettings()
    print("\n路径设置:")
    print(f"  项目包目录: {paths.package_root}")
    print(f"  配置目录: {paths.config_dir}")
    print(f"  资源目录: {paths.resources_dir}")
    print(f"  数据目录: {paths.data_dir}")




# client_http_client.py
import time
import http.client
from typing import Callable
import urllib.parse
import uuid

def _post(token: str, path: str, host: str="127.0.0.1", port: int=8000, data: dict={}):
    # 1. 序列化为 x-www-form-urlencoded
    body = urllib.parse.urlencode(data)
    # 2. 建立连接
    conn = http.client.HTTPConnection(host, port)
    # 3. 发送 POST
    conn.request(
        method="POST",
        url=path,
        body=body,
        headers={
            "Token": token,
            "Content-Type": "application/x-www-form-urlencoded"
        }
    )
    # 4. 获取响应
    resp = conn.getresponse()
    print(f"Status: {resp.status} {resp.reason}")
    text = resp.read().decode('utf-8', errors='ignore')
    print("Body:\n", text)
    conn.close()

def _start_inference_by_token(token: str, question: str):
    _post(token, "/start_inference", data={"question": question})

def _stop_inference_by_token(token: str, answer: str, is_correct: bool, is_valid: bool):
    _post(token, "/stop_inference", data={"answer": answer, "is_correct": is_correct, "is_valid": is_valid})

class Trace:
    def __init__(self):
        self.token = str(uuid.uuid4())

    def start_inference(self, question: str):
        _start_inference_by_token(self.token, question)

    def stop_inference(self, answer: str, is_correct: bool, is_valid: bool):
        _stop_inference_by_token(self.token, answer, is_correct, is_valid)

    def refresh_token(self):
        self.token = str(uuid.uuid4())
        return self.token
    
def determine_function(question: str, answer: str) -> bool:
    return True
def validate_function(question: str, answer: str) -> bool:
    return True

def tracker_with_correction(_determine_function: Callable[[str, str], bool]=determine_function, _validate_function: Callable[[str, str], bool]=validate_function):
    def decorator(func: Callable[[str], str]):
        trace = Trace()
        def wrapper(question: str):
            # 开始记录能耗
            trace.start_inference(question)
            
            # 执行原函数
            answer = func(question)
            
            # 判断是否正确以及合规
            is_correct = _determine_function(question,answer)
            is_valid = _validate_function(question,answer)
            # 停止记录能耗
            trace.stop_inference(answer, is_correct, is_valid)  
            
            return answer
        
        return wrapper

    return decorator






if __name__ == "__main__":
    @tracker_with_correction(determine_function, validate_function)
    def test_function(question: str) -> str:
        time.sleep(5)
        return "Hello, World!"
    # 示例调用
    while True:
        test_function("Hello?")
        time.sleep(10)







"""
使用rich结合原生logging实现命令行界面。
"""

import logging
from rich.console import Console
from rich.logging import RichHandler
import time
# ——1. 创建全局 Console（可自定义主题）
console = Console()

# ——2. 配置 RichHandler：对齐 level（8 列），显示时间，不显示文件路径
handler = RichHandler(
    console=console,
    show_time=True,
    log_time_format="%b %d %a %H:%M:%S",
    show_path=False     # 不要默认的 filename:lineno
)

# ——3. 基础 Logging 配置，只用 RichHandler
logging.basicConfig(
    level="DEBUG",
    format="%(message)s",  # 由 RichHandler 负责输出时间、级别等前缀
    handlers=[handler]
)
logger = logging.getLogger("app")

# ——4. 自定义 Adapter：注入“stage”到消息前，无对齐
class StageAdapter(logging.LoggerAdapter):
    def process(self, msg, kwargs):
        stage = self.extra.get("stage")
        if stage:
            msg = f"[{stage}] {msg}"
        return msg, kwargs

# ——5. 演示：在不同阶段记录日志
if __name__ == "__main__":
    init_log = StageAdapter(logger, {"stage": "INIT"})
    proc_log = StageAdapter(logger, {"stage": "PROCESS"})
    finish_log = StageAdapter(logger, {"stage": "FINISH"})

    proc_log.debug("Connecting to database")
    proc_log.info("Processing data")
    finish_log.warning("Low disk space")
    finish_log.error("Failed to write output")
    time.sleep(1)
    init_log.info("Loading configuration")
    proc_log.debug("Connecting to database")
    proc_log.info("Processing data")
    finish_log.warning("Low disk space")
    finish_log.error("Failed to write output")



from .config_reader import PATHS, load_config
# 加载全局配置
CONFIG = load_config()
# 导出路径设置为全局变量
PACKAGE_ROOT = PATHS.package_root
DATA_DIR = PATHS.data_dir
CONFIG_DIR = PATHS.config_dir
# 导出能源追踪配置为全局变量
ENABLE_CPU_MONITOR = CONFIG.energy_tracker_config.enable_cpu_power_monitor
ENABLE_GPU_MONITOR = CONFIG.energy_tracker_config.enable_gpu_power_monitor
USE_ACTUAL_CPU_POWER = CONFIG.energy_tracker_config.use_actual_cpu_power
USE_ACTUAL_GPU_POWER = CONFIG.energy_tracker_config.use_actual_gpu_power
MEASUREMENT_INTERVAL = CONFIG.energy_tracker_config.measurement_interval
CPU_POWER_BIAS = CONFIG.energy_tracker_config.cpu_power_bias
CPU_POWER_COEFFICIENT = CONFIG.energy_tracker_config.cpu_power_coefficient
GPU_POWER_BIAS = CONFIG.energy_tracker_config.gpu_power_bias
GPU_POWER_COEFFICIENT = CONFIG.energy_tracker_config.gpu_power_coefficient
